function [par_opt, chi_vector, chi_opt, SF_opt] = annealing_model_fit(par, D, SF, R, dt, d_rate, ind, T0, dT)

%INFO: annealing method to fit Unkel's data to our model

%INPUTS:
    %par: initial set of parameters
    %D: experimental dose values
    %SF: log of experimental surviving fractions
    %R: vector of doses rates per dt. To fit Unkel's data we will consider
        %a constant dose rate of 0.1 Gy/s (6 Gy/min)
    %dt: time step of the Euler method
    %d_rate: dose rate (scalar)
    %ind: indices of parameters in par that we want to optimize (ind=[1 2]
        %for LQ-like fits
    %T0: initial temperature for the annealing method
    %dT: cooling factor for the annealing method T_i+1=T_i*dT

%OUTPUTS:
    %par_opt: optimized parameters
    %chi_vector: evolution of the cost function
    %chi_opt: minimum_value of the cost function
    %SF_opt: log of the best-fitting surviving fractions
    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%indices of parameters to optimize (a and b in our model)


T=T0;
ind2=length(D);

N0=1e6; %initial number of cells (arbitrary)


for k=1:ind2
    
    TMAX=D(k)/d_rate+30;
    [N, ~, Ns] = model_dif_radionuclide_stochastic(D(k), R, dt, TMAX, par, 0);
 
    %surviving fraction (model)
    SF_th(k)=(N(end)+Ns(end))/N0;
    
end

SF_th=log(SF_th);
chi=sum(((SF_th-SF)).^2);
chi_opt=chi;
par_opt=par;
chi_vector=chi;
SF_opt=SF_th;

%START of simulated annealing

for i=1:500
    for j=1:500
        
        %perturbation of parameters
        k=fix(rand*length(ind)+1);
        k=ind(k);
        par_new=par;
        par_new(k)=par(k)*(1+(rand-0.5)*T/T0);
        
        for k=1:ind2
             TMAX=D(k)/d_rate+30;
             [N, ~, Ns] = model_dif_radionuclide_stochastic(D(k), R, dt, TMAX, par_new, 0);
             %surviving fraction (model)
             SF_th(k)=(N(end)+Ns(end))/N0;
        end
        
        SF_th=log(SF_th);
        chi_new=sum(((SF_th-SF)).^2);
        
        if oracle(chi_new-chi, T)==1
            chi=chi_new;
            par=par_new;
            chi_vector=[chi_vector chi_new];
            
            if chi_new<chi_opt
                chi_opt=chi_new;
                par_opt=par_new;
                SF_opt=SF_th;
            end
        end

    end
    T=T*dT;
end


end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%auxiliary function "oracle"
function valor = oracle(x,T)

    if x<0
        valor=1;
    else
        if rand<exp(-x/T)
            valor=1;
        else
            valor=0;
        end
    end

end