
function [N, Nd, Ns] = model_dif_radionuclide_stochastic(D0, R, dt, TMAX, par, stochasticity)


%INFO: stochastic implementation of the model

%INPUTS:
    %D0: total dose (Gy)
    %R: dose rate (Gy/s)
    %dt: time step of our numerical method (seconds)
    %TMAX: time limit of the simulation (seconds)
    %par: vector containing the parameters of our model
    %stochasticity: stochastic (=1) or deterministic (otherwise)
    %calculations

%OUTPUTS:
    %N: viable tumor cells versus time
    %Nd: doomed tumor cells versus time
    %Ns: sub-lethally damaged cells versus time

%asignation of parameters in "par"
a=par(1);
b=par(2);
lambda_1=par(3);
lambda_2=par(4);
lambda_3=par(5);
mu=par(6);
gamma=par(7);
%Michaelis-Menten
%kappa=par(8);


%initial numbers of cells
N(1)=1e6;
Nd(1)=0;
Ns(1)=0;


%if total dose is empty, we integrate the dose rate vector
if isempty(D0)
    D0=sum(R)*dt;
end
D=0;

%START OF ALGORITHM
t=0;
i=1;
while t<=TMAX
    
    if D<D0*0.99999
        d=R(i);
    else
        d=0;
    end
    
    D=D+d*dt;
    
    %viable tumor cells 
    %dN = lambda_1*N(i) + mu*Ns(i)/(kappa+Ns(i)) - a*d*N(i) -b*d*N(i);
    dN1=lambda_1*N(i)*(1-1e-6*(N(i)+Nd(i)+Ns(i)))*dt;
    dN2=a*d*N(i)*dt;
    dN3=b*d*N(i)*dt;
    
    %doomed tumor cells
    dNs1=a*d*Ns(i)*dt;
    dNs2=b*d*Ns(i)*dt;
    dNs3=lambda_3*Ns(i)*(1-1e-6*(N(i)+Nd(i)+Ns(i)))/dt;
    dNs4=mu*Ns(i)*dt;
    
    %sublethally damaged tumor cells
    dNd1=lambda_2*Nd(i)*(1-1e-6*(N(i)+Nd(i)+Ns(i)))*dt;
    dNd2=gamma*Nd(i)*dt;
    

    dN=(dN1+dNs4-dN2-dN3);
	dNs=(dN3-dNs4-dNs1-dNs2+dNs3);
	dNd=(dN2+dNs1+dNs2+dNd1-dNd2);

    if stochasticity == 1 
        dN=sign(dN)*poissrnd(sign(dN)*dN);
        dNs=sign(dNs)*poissrnd(sign(dNs)*dNs);
        dNd=sign(dNd)*poissrnd(sign(dNd)*dNd);
    end

    
    N(i+1)=N(i)+dN;
    Nd(i+1)=Nd(i)+dNd;
    Ns(i+1)=Ns(i)+dNs;
    
    if N(i+1)<0, N(i+1)=0;, end
    if Nd(i+1)<0, Nd(i+1)=0;, end
    if Ns(i+1)<0, Ns(i+1)=0;, end
    
    i=i+1;
    t=t+dt;
    
end


